<?php

/**
 * @file
 * Fan Courier administration page.
 */

/**
 * Fan Courier settings form.
 */
function commerce_shipping_fancourier_settings_form() {
  $form = array();

  $form['tab'] = array(
    '#type' => 'vertical_tabs',
    '#weight' => 99,
  );

  // Login data.
  $form['fancourier'] = array(
    '#type' => 'fieldset',
    '#title' => t('Fancourier creditentials'),
    '#group' => 'tab',
  );
  $fan = &$form['fancourier'];
  $fan['commerce_fancourier_username'] = array(
    '#type' => 'textfield',
    '#title' => 'Fan Curier "username"',
    '#default_value' => variable_get('commerce_fancourier_username', ''),
    '#required' => TRUE,
  );
  $fan['commerce_fancourier_user_pass'] = array(
    '#type' => 'textfield',
    '#title' => 'Fan Curier "user_pass"',
    '#default_value' => variable_get('commerce_fancourier_user_pass', ''),
    '#required' => TRUE,
  );
  $fan['commerce_fancourier_client_id'] = array(
    '#type' => 'textfield',
    '#title' => 'Fan Curier "client_id"',
    '#default_value' => variable_get('commerce_fancourier_client_id', ''),
    '#required' => TRUE,
  );

  // Fan Courier services from the API.
  $form['services'] = array(
    '#type' => 'fieldset',
    '#title' => t('Fancourier services'),
    '#group' => 'tab',
  );
  $serv = &$form['services'];
  $api_services = commerce_shipping_fancourier_get_services();
  // Enabled services in the admin UI.
  $saved_services = variable_get('commerce_fancourier_services', array());
  if ($api_services) {
    // Add every service with status and label. Fancourier class uses label.
    $serv['commerce_fancourier_services'] = array(
      '#type' => 'container',
      '#title' => t('Fan Courier services'),
      '#tree' => TRUE,
    );
    $services = &$serv['commerce_fancourier_services'];
    foreach ($api_services as $service_label) {
      $name = preg_replace('(--|<[^<>]+>|[^a-z0-9-])', '_', strtolower($service_label));
      $services[$name] = array(
        '#type' => 'container',
        '#tree' => TRUE,
      );
      $services[$name]['label'] = array(
        '#type' => 'value',
        '#value' => $service_label,
      );
      if (isset($saved_services[$name]['enable'])) {
        $enable = $saved_services[$name]['enable'];
      }
      else {
        $enable = 0;
      }
      $services[$name]['enable'] = array(
        '#type' => 'checkbox',
        '#title' => $service_label,
        '#default_value' => $enable,
      );
    }
  }
  else {
    $serv['info']['#markup'] = t('No services available.');
  }

  // Addressfield fieldset.
  $form['addressfield'] = array(
    '#type' => 'fieldset',
    '#title' => t('Addressfield configuration'),
    '#group' => 'tab',
  );
  // Customer profile reference field to shipping profile.
  $address = &$form['addressfield'];
  $field_type = 'commerce_customer_profile_reference';
  $entity_type = $bundle = 'commerce_order';
  $options = _commerce_shipping_fancourier_get_fields($field_type, $entity_type, $bundle);
  if (!empty($options)) {
    $address['commerce_fancourier_shipping_profile'] = array(
      '#type' => 'select',
      '#title' => t('Order shipping reference field'),
      '#description' => t('Choose the Commerce Customer profile type.'),
      '#options' => $options,
      '#required' => TRUE,
      '#default_value' => variable_get('commerce_fancourier_shipping_profile', 'commerce_customer_shipping'),
      '#ajax' => array(
        'callback' => '_commerce_shipping_fancourier_ajax_callback',
        'wrapper' => 'shipping-address',
      ),
    );
  }

  // Addressfield by shipping profile. Handle ajax calls. When another profile
  // reference is selected, check bundle settings in the profile type.
  $shipping_profile = variable_get('commerce_fancourier_shipping_profile', 'commerce_customer_shipping');
  if (isset($form_state['values']['commerce_fancourier_shipping_profile'])) {
    $shipping_profile = $form_state['values']['commerce_fancourier_shipping_profile'];
  }

  $bundle = 'shipping';
  if ($field = field_info_field($shipping_profile)) {
    $bundle = $field['settings']['profile_type'];
  }
  $field_type = 'addressfield';
  $entity_type = 'commerce_customer_profile';

  $address_options = _commerce_shipping_fancourier_get_fields($field_type, $entity_type, $bundle);
  $address['commerce_fancourier_shipping_addressfield'] = array(
    '#type' => 'select',
    '#title' => t('Address field'),
    '#description' => t('Choose the Commerce Customer profile type.'),
    '#options' => $address_options,
    '#required' => TRUE,
    '#default_value' => variable_get('commerce_fancourier_shipping_addressfield', 'commerce_customer_address'),
    '#prefix' => '<div id="shipping-address">',
    '#suffix' => '</div>',
  );

  return system_settings_form($form);
}

/**
 * Fan Courier settings form submit.
 */
function commerce_shipping_fancourier_settings_form_submit($form, $form_state) {
  // Reset shipping services to update active services.
  commerce_shipping_services_reset();
}

/**
 * Ajax callback for addressfield element.
 */
function _commerce_shipping_fancourier_ajax_callback($form, $form_state) {
  return $form['addressfield']['commerce_fancourier_shipping_addressfield'];
}

/**
 * Helper function to get fields.
 */
function _commerce_shipping_fancourier_get_fields($field_type, $entity_type, $bundle = NULL) {
  $fields = array();
  if ($bundle == NULL) {
    return $fields;
  }

  $all_fields = field_info_instances($entity_type, $bundle);
  if (empty($all_fields)) {
    return $fields;
  }
  foreach ($all_fields as $name => $info) {
    $field_info = field_info_field($name);
    if ($field_info['type'] == $field_type) {
      $fields[$name] = "{$info['label']} ({$name})";
    }
  }

  return $fields;
}
